{
  #!/bin/sh -e

  # ANSI color codes
  RED='\033[0;31m'
  CYAN='\033[0;36m'
  YELLOW='\033[0;33m'
  LIGHT_GREEN='\033[0;92m'
  BOLD='\033[1m'
  NC='\033[0m' # No Color

  echo_error() {
    printf "${BOLD}${RED}ERROR: ${NC}${RED}%s${NC}\n" "$1" >&2
  }

  echo_info() {
    printf "${BOLD}${CYAN}INFO: ${NC}${CYAN}%s${NC}\n" "$1"
  }

  echo_warning() {
    printf "${BOLD}${YELLOW}WARNING: ${NC}${YELLOW}%s${NC}\n" "$1"
  }

  echo_note() {
    printf "${BOLD}${LIGHT_GREEN}NOTE: ${NC}${LIGHT_GREEN}%s${NC}\n" "$1"
  }

  # ─< Check if the given command exists silently >─────────────────────────────────────────
  command_exists() {
    command -v "$@" >/dev/null 2>&1
  }

  # Check if the user is root and set sudo variable if necessary
  check_root() {
    if [ "$(id -u)" -ne 0 ]; then
      echo_note "checked $(id -u)"
      if command_exists sudo; then
        echo_note "Checking sudo"
        echo_warning "User is not root. Using sudo for privileged operations."
        _sudo="sudo"
      else
        echo_error "No sudo found and you're not root! Can't install packages."
        return 1
      fi
    else
      echo_info "Root access confirmed."
      _sudo=""
    fi
    if [ -n "$_sudo" ]; then
      echo_info "--- check_root done --- echo '$_sudo' |"
    else
      echo_warning "sudo variable is empty: $_sudo"
    fi
  }

  # ─< Dependencies >─────────────────────────────────────────────────────────────────────
  deps="git cargo meson luarocks pipx curl ripgrep make composer npm gcc g++ unzip zip"

  # ─< Distribution-specific installation functions >─────────────────────────────────────
  inst_ubuntu() {
    $_sudo apt-get update
    for _deps in $deps; do
      $_sudo apt-get install -y "$_deps" </dev/tty
    done
    inst_generic
  }

  inst_debian() {
    $_sudo apt-get update
    for _deps in $deps; do
      $_sudo apt-get install -y "$_deps" </dev/tty
    done
    inst_generic
  }

  inst_fedora() {
    $_sudo dnf update </dev/tty
    for _deps in $deps; do
      $_sudo dnf install -y "$_deps" </dev/tty
    done
    $_sudo dnf install -y neovim
  }

  inst_arch() {
    $_sudo pacman -Syu --noconfirm
    for _deps in $deps; do
      $_sudo pacman -S --noconfirm "$_deps"
    done
    $_sudo pacman -S --noconfirm --needet neovim
  }

  inst_opensuse() {
    $_sudo zypper refresh
    for _deps in $deps; do
      $_sudo zypper in "$_deps"
    done
    $_sudo zypper in neovim
  }

  inst_alpine() {
    $_sudo apk update
    for _deps in $deps; do
      $_sudo apk add "$_deps"
    done
    $_sudo apk add neovim
  }

  echo_info "Fetching latest stable Neovim version..."
  latest_version=$(curl -s https://api.github.com/repos/neovim/neovim/releases/latest | grep 'tag_name' | cut -d\" -f4)

  inst_generic() {
    tempdir="$(mktemp -d)"
    if [ ! -d "$HOME/.bin" ]; then
      mkdir -p "$HOME/.bin"
    fi

    echo_info "Downloading Neovim version $latest_version..."

    # Determine architecture
    arch=$(uname -m)
    if [ "$arch" = "aarch64" ] || [ "$arch" = "armv7l" ]; then
      nvim_tarball="nvim-linux-arm64.tar.gz"
    else
      nvim_tarball="nvim-linux-x86_64.tar.gz"
    fi

    # Download the tarball
    curl -fsSL -o "$tempdir/nvim.tar.gz" "https://github.com/neovim/neovim/releases/download/${latest_version}/${nvim_tarball}"

    # Change to the temporary directory and extract
    cd "$tempdir"
    tar -xzf nvim.tar.gz

    # Remove old installation if it exists
    rm -rf "$HOME/.bin/nvim-linux-x86_64" "$HOME/.bin/nvim-linux-arm64"

    # Copy to destination
    cp -r ./nvim-linux-* "$HOME/.bin/"

    # Create symlinks with absolute paths
    inst_paths="/usr/bin /usr/share/bin $HOME/.local/bin"
    for in_path in $inst_paths; do
      if [ -d "$in_path" ]; then
        echo_info "Installing into $in_path"
        $_sudo ln -sf "$HOME/.bin/nvim-linux-*/bin/nvim" "$in_path/nvim"
      fi
    done

    # Clean up
    rm -rf "$tempdir"

    echo_info "Neovim $latest_version installed successfully"
  }

  checkVersion() {
    currentVersion="$(nvim --version | head -n1 | awk '{print $2}')"
  }

  # ─< Distribution detection and installation >────────────────────────────────────────
  get_packager() {
    if [ -e /etc/os-release ]; then
      echo_info "Detecting distribution..."
      . /etc/os-release

      # Convert $ID and $ID_LIKE to lowercase
      ID=$(printf "%s" "$ID" | tr '[:upper:]' '[:lower:]')
      ID_LIKE=$(printf "%s" "$ID_LIKE" | tr '[:upper:]' '[:lower:]')

      case "$ID" in
      ubuntu | pop) inst_ubuntu ;;
      debian) inst_debian ;;
      fedora) inst_fedora ;;
      alpine) inst_alpine ;;
      arch | manjaro | garuda | endeavour) inst_arch ;;
      opensuse*) inst_opensuse ;;
      *)
        if [ "${ID_LIKE#*debian}" != "$ID_LIKE" ]; then
          inst_debian
        elif [ "${ID_LIKE#*ubuntu}" != "$ID_LIKE" ]; then
          inst_ubuntu
        elif [ "${ID_LIKE#*arch}" != "$ID_LIKE" ]; then
          inst_arch
        elif [ "${ID_LIKE#*fedora}" != "$ID_LIKE" ]; then
          inst_fedora
        elif [ "${ID_LIKE#*suse}" != "$ID_LIKE" ]; then
          inst_opensuse
        else
          echo_error "Unsupported distribution: $ID"
          exit 1
        fi
        ;;
      esac
    else
      echo_error "Unable to detect distribution. /etc/os-release not found."
      exit 1
    fi
  }

  # ─< Main function >─────────────────────────────────────────────────────────────────
  main() {
    if command_exists nvim; then
      checkVersion
      if [ "$(printf '%s\n' "$latest_version" "$currentVersion" | sort -V | head -n1)" = "$latest_version" ]; then
        echo_warning "Neovim is already installed in version: $currentVersion"
        echo_error "Exiting now!"
        exit 0
      fi
    fi

    echo_info "Starting Neovim installation script..."
    get_packager
    if command -v nvim >/dev/null 2>&1; then
      echo_note "Neovim has been successfully installed!"
      echo_info "Neovim version: $(nvim --version | head -n1)"
    else
      echo_error "Neovim installation failed."
      exit 1
    fi
  }

  # ─< Script execution >─────────────────────────────────────────────────────────────
  check_root && main
}
